@@ -129,9 +129,10 @@ setup (Fixture *f,
   dbus_error_init (&f->e);
   g_queue_init (&f->messages);
 
-  if ((g_str_has_prefix (address, "tcp:") ||
-       g_str_has_prefix (address, "nonce-tcp:")) &&
-      !test_check_tcp_works ())
+  if ((g_str_has_prefix (address, "unix:") && !test_check_af_unix_works ()) ||
+      ((g_str_has_prefix (address, "tcp:") ||
+        g_str_has_prefix (address, "nonce-tcp:")) &&
+       !test_check_tcp_works ()))
     {
       f->skip = TRUE;
       return;
@@ -338,6 +339,12 @@ main (int argc,
     char **argv)
 {
   int ret;
+#ifdef DBUS_UNIX
+  char *tmp = _dbus_strdup ("/tmp");
+#else
+  char *tmp = dbus_address_escape_value (g_get_tmp_dir ());
+#endif
+  gchar *unix_tmpdir = g_strdup_printf ("unix:tmpdir=%s", tmp);
 
   test_init (&argc, &argv);
 
@@ -348,16 +355,16 @@ main (int argc,
   g_test_add ("/limit/tcp", Fixture, "tcp:host=127.0.0.1", setup,
       test_limit, teardown);
 
-#ifdef DBUS_UNIX
-  g_test_add ("/connect/unix", Fixture, "unix:tmpdir=/tmp", setup,
+  g_test_add ("/connect/unix", Fixture, unix_tmpdir, setup,
       test_connect, teardown);
-  g_test_add ("/relay/unix", Fixture, "unix:tmpdir=/tmp", setup,
+  g_test_add ("/relay/unix", Fixture, unix_tmpdir, setup,
       test_relay, teardown);
-  g_test_add ("/limit/unix", Fixture, "unix:tmpdir=/tmp", setup,
+  g_test_add ("/limit/unix", Fixture, unix_tmpdir, setup,
       test_limit, teardown);
-#endif
 
   ret = g_test_run ();
   dbus_shutdown ();
+  g_free (unix_tmpdir);
+  dbus_free (tmp);
   return ret;
 }
