@@ -2,6 +2,7 @@
 /* dbus-monitor.c  Utility program to monitor messages on the bus
  *
  * Copyright (C) 2003 Philip Blundell <philb@gnu.org>
+ * SPDX-License-Identifier: GPL-2.0-or-later
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +22,6 @@
 
 #include <config.h>
 
-#include "dbus/dbus-connection-internal.h"
 #include "dbus/dbus-internals.h"
 
 #include <stdio.h>
@@ -495,7 +495,7 @@ main (int argc, char *argv[])
   /* Receive o.fd.Peer messages as normal messages, rather than having
    * libdbus handle them internally, which is the wrong thing for
    * a monitor */
-  _dbus_connection_set_builtin_filters_enabled (connection, FALSE);
+  dbus_connection_set_builtin_filters_enabled (connection, FALSE);
 
   if (!dbus_connection_add_filter (connection, filter_func,
                                    _DBUS_INT_TO_POINTER (binary_mode), NULL))
